# Generated by Django 3.1.8 on 2021-05-18 09:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_twined", "0002_slug_unique_and_not_editable"),
    ]

    operations = [
        migrations.RenameField(
            model_name="abstractregisteredservice",
            old_name="slug",
            new_name="name",
        ),
    ]
